using System;
using System.Security.Cryptography;
using System.Text;

class RSAVerschluesselungBeispiel
{
    static void Main()
    {
        try
        {
            // Erzeugen eines ASCIIEncoders zur Umwandlung zwischen Byte-Array und String
            ASCIIEncoding ByteConverter = new ASCIIEncoding();

            // Der zu verschlsselnde Klartext als Byte-Array
            byte[] klartext = ByteConverter.GetBytes("Dieser Text ist streng geheim!");
             
            // Erzeugen einer RSACryptoServiceProvider-Instanz samt Schlsselpaar
            RSACryptoServiceProvider rsaAlg = new RSACryptoServiceProvider();

            // Schlssel in eigenen Variablen speichern
            // (dient nur der bersichtlichkeit)
            RSAParameters RSApublicKey    = rsaAlg.ExportParameters(false);
            RSAParameters RSAprivateKey = rsaAlg.ExportParameters(true);

            // Verschlsseln des Klartextes mit dem ffentlichen Schlssel
            byte[] schluesseltext = RSAVerschluessele(klartext, RSApublicKey);

            // Entschlsseln des Schlsseltextes mit dem privaten Schlssel
            byte[] entschluesselt = RSAEntschluessele(schluesseltext, RSAprivateKey);

            // Ausgabe des entschlsselten Schlsseltextes
            Console.WriteLine("Klartext:      {0}", ByteConverter.GetString(klartext));
            Console.WriteLine("Entschlsselt: {0}", ByteConverter.GetString(entschluesselt));

            // Vertrauliche Daten lschen
            rsaAlg.Clear();
        }
        catch (Exception e)
        {
            Console.WriteLine("Fehler: {0}", e.Message);
        }
    }

    static public byte[] RSAVerschluessele(byte[] klartext, RSAParameters schluessel)
    {
        try
        {     
            // Erzeugen einer neuen Instanz von RSACryptoServiceProvider
            RSACryptoServiceProvider rsaAlg = new RSACryptoServiceProvider();

            // Importieren des bergebenen ffentlichen Schlssels
            rsaAlg.ImportParameters(schluessel);

            // Verschlsseln der bergebenen Daten
            // Der Boolean-Wert besagt, das kein OAEP-Padding erfolgt
            byte[] schluesseltext = rsaAlg.Encrypt(klartext, false);

            // Vertrauliche Daten lschen
            rsaAlg.Clear();

            // Schlsseltext zurckgeben
            return schluesseltext;
        }
        catch (Exception e)
        {
            Console.WriteLine("Fehler beim Verschlsseln: {0}", e.Message);
            return null;
        }
    }

    static public byte[] RSAEntschluessele(byte[] schluesseltext, RSAParameters schluessel)
    {
        try
        {
            // Erzeugen einer neuen Instanz von RSACryptoServiceProvider
            RSACryptoServiceProvider rsaAlg = new RSACryptoServiceProvider();

            // Importieren des bergebenen privaten Schlssels
            rsaAlg.ImportParameters(schluessel);

            // Entschlsseln der bergebenen Daten
            // Der Boolean-Wert besagt, das kein OAEP-Padding erfolgt
            byte[] entschluesselt = rsaAlg.Decrypt(schluesseltext, false);

            // Vertrauliche Daten lschen
            rsaAlg.Clear();

            // Schlsseltext zurckgeben
            return entschluesselt;
        }
        catch (Exception e)
        {
            Console.WriteLine("Fehler beim Entschlsseln: {0}", e.Message);
            return null;
        }
    }
}